@@ -1,14 +1,19 @@
 package gc
 
 import (
+	"context"
 	"os"
 	"path/filepath"
 	"sync"
 	"testing"
 	"time"
 
+	"github.com/open-cluster-management/multicloud-operators-foundation/pkg/utils"
+	rbacv1 "k8s.io/api/rbac/v1"
+	k8sfake "k8s.io/client-go/kubernetes/fake"
+
 	"github.com/onsi/gomega"
-	actionv1beta1 "github.com/stolostron/multicloud-operators-foundation/pkg/apis/action/v1beta1"
+	actionv1beta1 "github.com/open-cluster-management/multicloud-operators-foundation/pkg/apis/action/v1beta1"
 	"github.com/stretchr/testify/assert"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
@@ -66,17 +71,17 @@ func TestMain(m *testing.M) {
 }
 
 // StartTestManager adds recFn
-func StartTestManager(mgr manager.Manager, g *gomega.GomegaWithT) (chan struct{}, *sync.WaitGroup) {
-	stop := make(chan struct{})
+func StartTestManager(mgr manager.Manager, g *gomega.GomegaWithT) (context.CancelFunc, *sync.WaitGroup) {
+	ctx, cancel := context.WithCancel(context.Background())
 	wg := &sync.WaitGroup{}
 	wg.Add(1)
 
 	go func() {
 		defer wg.Done()
-		g.Expect(mgr.Start(stop)).NotTo(gomega.HaveOccurred())
+		g.Expect(mgr.Start(ctx)).NotTo(gomega.HaveOccurred())
 	}()
 
-	return stop, wg
+	return cancel, wg
 }
 
 func TestControllerReconcile(t *testing.T) {
@@ -91,10 +96,10 @@ func TestControllerReconcile(t *testing.T) {
 
 	SetupWithManager(mgr)
 
-	stopMgr, mgrStopped := StartTestManager(mgr, g)
+	cancel, mgrStopped := StartTestManager(mgr, g)
 
 	defer func() {
-		close(stopMgr)
+		cancel()
 		mgrStopped.Wait()
 	}()
 
@@ -117,6 +122,7 @@ func validateError(t *testing.T, err, expectedErrorType error) {
 }
 
 func TestReconcile(t *testing.T) {
+	ctx := context.Background()
 	tests := []struct {
 		name              string
 		existingObjs      []runtime.Object
@@ -217,9 +223,47 @@ func TestReconcile(t *testing.T) {
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
 			svrc := newTestReconciler(test.existingObjs)
-			res, err := svrc.Reconcile(test.req)
+			res, err := svrc.Reconcile(ctx, test.req)
 			validateError(t, err, test.expectedErrorType)
 			assert.Equal(t, res.Requeue, false)
 		})
 	}
 }
+
+var objs = []runtime.Object{
+	&rbacv1.ClusterRole{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "clusterrole1",
+			Finalizers: []string{
+				ClustersetFinalizerName,
+			},
+		},
+		Rules: nil,
+	},
+	&rbacv1.ClusterRole{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "clusterrole2",
+			Finalizers: []string{
+				ClustersetFinalizerName,
+			},
+		},
+		Rules: nil,
+	},
+}
+
+func TestClean(t *testing.T) {
+	kubeClient := k8sfake.NewSimpleClientset(objs...)
+	ctx := context.Background()
+	gf := NewCleanGarbageFinalizer(kubeClient)
+	gf.clean()
+	clusterrole1, err := kubeClient.RbacV1().ClusterRoles().Get(ctx, "clusterrole1", metav1.GetOptions{})
+	validateError(t, err, nil)
+	if utils.ContainsString(clusterrole1.GetFinalizers(), ClustersetFinalizerName) {
+		t.Errorf("Finalizer is not removed, clusterrole: %v", clusterrole1)
+	}
+	clusterrole2, err := kubeClient.RbacV1().ClusterRoles().Get(ctx, "clusterrole1", metav1.GetOptions{})
+	validateError(t, err, nil)
+	if utils.ContainsString(clusterrole2.GetFinalizers(), ClustersetFinalizerName) {
+		t.Errorf("Finalizer is not removed")
+	}
+}
